// Copyright (C) 1999 Open Source Telecom Corporation.
//  
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software 
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
// 
// As a special exception to the GNU General Public License, permission is 
// granted for additional uses of the text contained in its release 
// of APE.
// 
// The exception is that, if you link the APE library with other files
// to produce an executable, this does not by itself cause the
// resulting executable to be covered by the GNU General Public License.
// Your use of that executable is in no way restricted on account of
// linking the APE library code into it.
// 
// This exception does not however invalidate any other reasons why
// the executable file might be covered by the GNU General Public License.
// 
// This exception applies only to the code released under the 
// name APE.  If you copy code from other releases into a copy of
// APE, as the General Public License permits, the exception does
// not apply to the code that you add in this way.  To avoid misleading
// anyone as to the status of such modified files, you must delete
// this exception notice from them.
// 
// If you write modifications of your own for APE, it is your choice
// whether to permit this exception to apply to your modifications.
// If you do not wish that, delete this exception notice.  

#ifndef	__APE_SERIAL_H__
#define	__APE_SERIAL_H__

#ifndef	__APE_FILE_H__
#include <APE/file.h>
#endif

#include <termios.h>

#define	MAX_SERIAL_TIMEOUT	((timeout_t)10000)

typedef enum
{
	SERIAL_FLOW_NONE,
	SERIAL_FLOW_HARD,
	SERIAL_FLOW_SOFT,
	SERIAL_FLOW_BOTH
};

typedef	enum
{
	SERIAL_FORMAT_7E1,
	SERIAL_FORMAT_7O1,
	SERIAL_FORMAT_7N1,
	SERIAL_FORMAT_7S2,
	SERIAL_FORMAT_8E1,
	SERIAL_FORMAT_8O1,
	SERIAL_FORMAT_8N1,
	SERIAL_FORMAT_8S2
};

class Serial
{
private:
	int _fd;
	struct termios _restore;
	struct termios _current;
	char _lockname[65];
	char _nl;

public:
	Serial(const char *fname, long speed, int format, int flow);
	virtual ~Serial();	

	void setFlowControl(int flow);
	void setSpeed(long speed);
	void setFormat(int format);
	int  Inkey(timeout_t timer = 0);
	int Read(void *buf, size_t len, timeout_t timer = 0);
	int Readline(char *buf, size_t max, timeout_t timer = 0);

	inline void setNewline(char nl)
		{_nl = nl;};

	inline virtual int Write(void *buf, size_t len)
		{return write(_fd, (char *)buf, len);};

	friend inline int write(Serial &s, void *buf, size_t len)
		{return write(s._fd, (char *)buf, len);};

	inline virtual int Write(char *buf)
		{return write(_fd, buf, strlen(buf));};

	friend inline int write(Serial &s, char *buf)
		{return write(s._fd, buf, strlen(buf));};
};

inline int inkey(Serial &s, timeout_t timer = 0)
	{return s.Inkey(timer);};

inline int read(Serial &s, void *buf, size_t len, timeout_t timer = 0)
	{return s.Read(buf, len, timer);};

inline int readline(Serial &s, char *buf, size_t max, timeout_t timer = 0)
	{return s.Readline(buf, max, timer);};
#endif
